/*
' DotNetNuke - http:'www.dotnetnuke.com
' Copyright (c) 2002-2007
' by DotNetNuke Corporation
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
 */

/*
* FILE:        WindowsLiveLogin.cs
*                                                                      
* DESCRIPTION: Sample implementation of Auth Lite protocol in C#.
*
* HISTORY:     16 April 2007 - Mix '07 Version, Subject To Change.
*
* Copyright <cp> 2007 Microsoft Corporation.  All Rights Reserved.
*/

using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections.Specialized;
using System.Collections;
using System.Web;
using System.Web.Configuration;
using System.Security.Cryptography;
using System.IO;
using System.Net;
using System.Reflection;
using System.Drawing;

namespace DotNetNuke.Authentication.LiveID
{
    /// <summary>
    /// Holds the user information after a successful login.
    /// </summary>
    /// <list type="table">
    ///  <item><term>timestamp</term> <description>is the time as
    ///  obtained from the SSO token.</description></item>
    ///
    ///  <item><term>id</term> <description>is the pairwise unique
    ///  ID for the user.</description></item>
    ///
    ///  <item><term>context</term> <description>is the
    ///  application context that was originally passed to the
    ///  login request, if any.</description>
    ///
    ///  <item><term>token</term> <description>is the encrypted
    ///  AuthLite token containing the UID. This can be cached in
    ///  a cookie and the UID can be retrieved by calling the
    ///  processToken method.</description>
    /// </list>
    public class User
    {
        string id;
        private string context;
        private DateTime timestamp;
        private string token;
        private bool usePersistentCookie;

        #region Constructors

        /// <summary>
        /// Creates a User object.
        /// </summary>
        public User(string timestamp, string id, string flags, string context, string token)
        {
            setTimestamp(timestamp);
            setId(id);
            setFlags(flags);
            setContext(context);
            setToken(token);
        }

        #endregion

        public bool IsSane()
        {
            if (Timestamp == null || Id == null)
                return false;

            try
            {
                Convert.ToInt32(Timestamp);
            }
            catch (Exception)
            {
                return false;
            }

            Regex re = new Regex("^\\w+$");
            if (!re.IsMatch(Id))
                return false;

            return true;
        }

        #region Public Properties

        /// <summary>
        /// Context is the application context that was originally
        /// passed to the login request, if any.
        /// </summary>
        public string Context { get { return context; } }

        /// <summary>
        /// Id is the pairwise unique ID for the user.
        /// </summary>
        public string Id { get { return id; } }

        /// <summary>
        ///  Timestamp is the time as obtained from the SSO token.
        /// </summary>
        public DateTime Timestamp { get { return timestamp; } }

        /// <summary>
        /// Token is the encrypted Web Authentication token containing
        /// the UID. This can be cached in a cookie and the UID can be
        /// retrieved by calling the ProcessToken method.
        /// </summary>
        public string Token { get { return token; } }

        /// <summary>
        /// Indicates whether the application
        /// is expected to store the user token in a session or
        /// persistent cookie.
        /// </summary>
        public bool UsePersistentCookie { get { return usePersistentCookie; } }

        #endregion

        #region Private Methods

        private void setContext(string context)
        {
            this.context = context;
        }

        private void setFlags(string flags)
        {
            this.usePersistentCookie = false;
            if ((flags != null) && (flags != ""))
            {
                try
                {
                    int flagsInt = Convert.ToInt32(flags);
                    this.usePersistentCookie = ((flagsInt % 2) == 1);
                }
                catch (Exception)
                {
                    throw new ArgumentException("Error: User: Invalid flags: "
                                                + flags);
                }
            }
        }

        private void setId(string id)
        {
            if (id == null)
            {
                throw new ArgumentException("Error: User: Null id in token.");
            }

            Regex re = new Regex(@"^\w+$");
            if (!re.IsMatch(id))
            {
                throw new ArgumentException("Error: User: Invalid id: " + id);
            }

            this.id = id;
        }

        private void setTimestamp(string timestamp)
        {
            if (timestamp == null)
            {
                throw new ArgumentException("Error: User: Null timestamp in token.");
            }

            int timestampInt;

            try
            {
                timestampInt = Convert.ToInt32(timestamp);
            }
            catch (Exception)
            {
                throw new ArgumentException("Error: User: Invalid timestamp: "
                                            + timestamp);
            }

            DateTime refTime = new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc);
            this.timestamp = refTime.AddSeconds(timestampInt);
        }

        private void setToken(string token)
        {
            this.token = token;
        }

        #endregion

    }

}
